/**
 * JMUD for JamochaMUD is the "main" section for JamochaMUD
 * $Id: JMUD.java,v 1.5 2006/11/17 06:23:41 jeffnik Exp $
 */

/* JamochaMUD, a Muck/Mud client program
 * Copyright (C) 1998-2005 Jeff Robinson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package anecho.JamochaMUD;

import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;

import anecho.gui.JSyncFrame;
import anecho.gui.SyncFrame;
import anecho.gui.SyncFrameGroup;

public class JMUD {
    
    public static CHandler connections;  // This class will handle all our connections
    public static volatile JMConfig settings;  // This class contains all our settings
    public MuckMain mainProg;          // Our main text window
    public Frame mainFrame;
    public DataIn inputFrame;
    public EnumPlugIns plugEnum;
    public SyncFrameGroup frameGroup;   // Our SyncFrame group
    private Object splash;
    private boolean useSwing = false;
    private static final boolean DEBUG = false;
    
    /**
     * The main method for JamochaMUD.  This is where it all begins!
     * @param args
     */
    public static void main(final String[] args) {
        // Create a new instance of our program for your running pleasure!
        // final JMUD proggy = new JMUD(args);
        new JMUD(args);
        // Bye-bye
    }
    
    /**
     *
     * @param args
     */
    public JMUD(String[] args) {
        
        // Create our "settings" container
        settings = JMConfig.getInstance();
        
        // Let's put ourselves in the settings, so we're accessable for a QUIT or somethin'
        settings.setJMCore(this);
        
        // Set some defaults
        settings.setJMValue(JMConfig.USESWING, true);  // Use Swing components by default
        settings.setJMValue(JMConfig.USESWINGENTRY, true);  // Java 1.2 and 1.3 requestFocus is "broken"
        settings.setJMValue(JMConfig.BGPAINT, false);    // Paint background on Swing text component
        
        // Check to see if we're forcing "single user mode" or not
        if (args.length > 0) {
            String tempArg;
            for (int i = 0; i < args.length; i++) {
                tempArg = args[i].toLowerCase();
                if (tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-h")) || tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-?"))) {
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("JamochaMUD_") + AboutBox.verNum);
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("A_Java_MUD/Muck_client,_(C)2003_Jeff_Robinson."));
                    System.out.println();
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Current_accepted_arguments:"));
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-h/-?_____This_screen"));
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-s________Force_single-user_mode"));
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-swing____Use_Java2_Swing_components_(default)"));
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-noswing__Use_only_Java_1.1.x_AWT_components"));
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-nosentry_Use_Swing_except_for_the_text-entry_area"));
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-nobg_____Do_not_paint_text_background_colours_(default)"));
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-bg_______Paint_text_background_colours"));
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-v________Version_of_the_program"));
                    System.exit(0);
                }
                
                if (tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-s"))) {
                    // Force single user mode
                    settings.setJMValue(JMConfig.SINGLEUSERMODE, true);
                }
                
                if (tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-v"))) {
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("JamochaMUD_") + AboutBox.verNum);
                    System.exit(0);
                }
                
                if (tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-swing"))) {
                    settings.setJMValue(JMConfig.USESWING, true);
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-swing_option_is_deprecated..._enabled_by_default."));
                }
                
                if (tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-noswing"))) {
                    settings.setJMValue(JMConfig.USESWING, false);
                    settings.setJMValue(JMConfig.USESWINGENTRY, false);
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Disabling_use_of_Swing_components."));
                }
                
                if (tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-nosentry"))) {
                    settings.setJMValue(JMConfig.USESWINGENTRY, false);
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Using_AWT_for_the_text_entry_area."));
                }
                
                if (tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-sentry"))) {
                    settings.setJMValue(JMConfig.USESWINGENTRY, true);
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Forcing_Swing-based_text-entry_component."));
                }
                
                if (tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-nobg"))) {
                    settings.setJMValue(JMConfig.BGPAINT, false);
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Background_text_painting_disabled."));
                }
                
                if (tempArg.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("-bg"))) {
                    settings.setJMValue(JMConfig.BGPAINT, true);
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Background_text_painting_enabled."));
                }
            }
        }
        
        // Test the system to see if Swing is available (if chosen)
        useSwing = settings.getJMboolean(JMConfig.USESWING);
        
        if (useSwing) {
            try {
                // Class swingTest = Class.forName("javax.swing.JButton");
                Class.forName(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("javax.swing.JButton"));
                if (DEBUG) {
                    System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Swing_test_passed.."));
                }
                
            } catch (Exception except) {
                System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Swing_is_not_currently_available,_switching_to_use_the_Java_AWT."));
                if (DEBUG) {
                    System.err.println(except);
                }
                
                settings.setJMValue(JMConfig.USESWING, false);
                settings.setJMValue(JMConfig.USESWINGENTRY, false);
                useSwing = false;
            }
        }
        
        // Create our SplashScreen
        if (useSwing) {
            splash = (Object)new anecho.gui.SplashScreen(null, java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Getting_system_information"), new javax.swing.ImageIcon(JMainMenu.class.getResource(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("JMUDSplash.png"))));
            ((anecho.gui.SplashScreen)splash).setVisible(true);
        }
        
        // Get our system information
        getSysInfo();
        
        // Read in our "profile"
        if (useSwing) {
            ((anecho.gui.SplashScreen)splash).updateMessage(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Reading_JamochaMUD_settings..."));
        }
        getSettings();
        
        frameGroup = new SyncFrameGroup();
        
        if (useSwing) {
            ((anecho.gui.SplashScreen)splash).updateMessage(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Loading_main_program..."));
        }
        
        mainProg = MuckMain.getInstance();
        mainFrame = settings.getJMFrame(JMConfig.MUCKMAINFRAME);
        frameGroup.add(mainFrame);
        // settings.setMainWindowVariable(mainProg);
        
        // connections = new CHandler(mainProg);
        connections = new CHandler();
        settings.setConnectionHandler(connections);
        
        inputFrame = new DataIn();
        frameGroup.add(inputFrame);
        settings.setDataInVariable(inputFrame);
        
        
        // Set the visual state of our components
        setupFrames();
        
        // Add our text-window to the main frame
        mainProg.setMainLayout();
        
        if (useSwing) {
            ((anecho.gui.SplashScreen)splash).updateMessage(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Loading_plug-ins..."));
        }
        loadPlugins();
        
        if (useSwing) {
            ((anecho.gui.SplashScreen)splash).setVisible(false);
        }
        
        if (DEBUG) {
            System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("JMUD.JMUD_is_calling_connectToNewMU()"));
        }
        connections.connectToNewMU();
        
    }
    
    /** Read in our settings.  We make this a separate method
     * in hopes that it will fall out of scope and destroy our
     * JMParseRC class when it is done... */
    private void getSettings() {
        // We'll pass our 'settings' variable to get things set up correctly
        new JMParseRC(splash);
    }
    
    /** Gather system-specific information about the platform
     * we're running on */
    private void getSysInfo() {
        new SysInfo();
    }
    
    /** Setup the visual elements for the user */
    private void setupFrames() {
        settings.setJMValue(JMConfig.MUCKMAINTITLE, java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("JamochaMUD"));
        
        // Set the size of our windows
        Rectangle tempRect;
        tempRect = settings.getJMRectangle(JMConfig.MAINWINDOW);
        
        if (tempRect != null) {
            mainFrame.setBounds(settings.getJMRectangle(JMConfig.MAINWINDOW));
        } else {
            mainFrame.setBounds(new Rectangle(0, 10, 600, 355));
        }
        
        tempRect = settings.getJMRectangle(JMConfig.DATABAR);
        
        if (tempRect != null) {
            inputFrame.setBounds(settings.getJMRectangle(JMConfig.DATABAR));
        } else {
            inputFrame.setBounds(new Rectangle(0, 385, 600, 60));
        }
        
        
        boolean sync = false;
        if (settings.getJMboolean(JMConfig.SPLITVIEW)) {
            sync = settings.getJMboolean(JMConfig.SYNCWINDOWS);
        }
        
        // Do this for now, but fix the setGroupSync function in SyncFrame/SyncGroup!!!
        // Fix me XXX
        // ((SyncFrame)mainFrame).setSyncFrameGroup(frameGroup);
        inputFrame.setSyncFrameGroup(frameGroup);
        // ((SyncFrame)mainFrame).setSync(sync);
        inputFrame.setSync(sync);
        
        // Create our area that will hold our output "windows"
        if (settings.getJMboolean(JMConfig.USESWING)) {
            if (DEBUG) {
                System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Swing_being_used_by_JMUD."));
            }
            ((JSyncFrame)mainFrame).setSyncFrameGroup(frameGroup);
            ((JSyncFrame)mainFrame).setSync(sync);
        } else {
            if (DEBUG) {
                System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("We're_not_using_swing_in_JMUD."));
            }
            // anecho.JamochaMUD.legacy.JMTabPanel textPanel = new anecho.JamochaMUD.legacy.JMTabPanel();
            ((SyncFrame)mainFrame).setSyncFrameGroup(frameGroup);
            ((SyncFrame)mainFrame).setSync(sync);
        }
        
    }
    
    private void loadPlugins() {
        plugEnum = EnumPlugIns.getInstance();
        settings.setPlugEnumerator(plugEnum);
        plugEnum.loadPlugIns(splash);
    }
    
    /** Tell our plug-ins that we are shutting down, so they better save their data! */
    private void haltPlugins() {
        plugEnum.haltPlugIns();
    }
    
    /** This method collects all the data needed to write
     * a new .rc file of the user's settings, and do
     * any other necessary clean-up */
    public void quitJamochaMUD() {
        // Gather all our settings that remain.
        Rectangle test;
        
        test = mainFrame.getBounds();
        if (checkVisible(test)) {
            settings.setJMValue(JMConfig.MAINWINDOW, test);
            
        }
        
        test = inputFrame.getBounds();
        if (checkVisible(test)) {
            settings.setJMValue(JMConfig.DATABAR, test);
        }
        
        // final JMWriteRC done = new JMWriteRC(settings);
        final JMWriteRC done = new JMWriteRC();
        done.writeRCToFile();
        
        haltPlugins();
        
        System.exit(0);
    }
    
    /** Check the bounds of our rectangle to see if it is visible
     * or not.  This is basically a hack, but better'n nothing.
     */
    private boolean checkVisible(final Rectangle testRect) {
        int xAxis, yAxis;   // Our location on the screen
        boolean retVal = true;
        xAxis = testRect.x;
        yAxis = testRect.y;
        
        if (xAxis < 0 || yAxis < 0) {
            // Our object is probably minimised, so use the settings from JMConfig
            retVal = false;
        }
        
        // Our object is probably visible on-screen, so we'll return true
        return retVal;
    }
    
    /**
     * Change our fonts globally across all windows
     * @param newStyle
     */
    public void setAllFonts(final Font newStyle) {
        // Change the font style of all our connections
        connections.setAllFonts(newStyle);
        
        // Change the font style of the input bar
        inputFrame.setFont(newStyle);
    }
    
    /**
     * Change our colours globally across all windows
     * @param foreground
     * @param background
     */
    public void setAllColours(final Color foreground, final Color background) {
        // Change the colours of our connections
        connections.setAllColours(foreground, background);
        
        // set the colour of our input frame
        inputFrame.setBackgroundColour(background);
        inputFrame.setForegroundColour(foreground);
    }
    
    /**
     * Change the synchronisation status of our windows
     * @param sync
     */
    public void setAllSync(final boolean sync) {
        // Only change our Sync value if we're using split panes
        if (settings.getJMboolean(JMConfig.SPLITVIEW)) {
            settings.setJMValue(JMConfig.SYNCWINDOWS, sync);
            ((SyncFrame)mainFrame).setSync(sync);
            inputFrame.setSync(sync);
        }
    }
    
    /** Change the main title */
    public void setMainWindowTitle() {
        mainProg.setWindowTitle();
        inputFrame.setWindowTitle();
    }
}
