/**
 * A MuSocket is used to hold all information important to
 * a single socket connection
 * $Id: MuSocket.java,v 1.8 2007/01/29 04:17:16 jeffnik Exp $
 */

/* JamochaMUD, a Muck/Mud client program
 * Copyright (C) 1998-2007 Jeff Robinson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package anecho.JamochaMUD;

import anecho.gui.JMText;
import anecho.gui.JMSwingText;

import anecho.extranet.Socks5socket;
import anecho.extranet.MUDBufferedReader;
import anecho.extranet.event.TelnetEvent;
import anecho.extranet.event.TelnetEventListener;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import java.io.DataOutputStream;

import java.net.ConnectException;
import java.net.InetAddress;

import java.net.NoRouteToHostException;
import java.net.Socket;
import java.net.UnknownHostException;

import java.util.Vector;



public class MuSocket extends Thread implements MouseListener, TelnetEventListener {
    
    
    
    private String muckName, address;
    private int port;
    private boolean ssl;
    private long timeStamp;
    private int activity = 0;   // Keep track of new activity if we're not the activeMU
    private boolean activeStatus = true;   // Is this connection the "active" one?
    private boolean connected = false;        // The state of the connection
    private boolean paused;             // Has our output been paused?
    private boolean echo = true;        // MU*'s should always start with ECHO on.
    private JMText mainText;            // The text-area belonging to this connection
    private JMSwingText mainSText;      // The swing version of our text area
    private JMConfig settings;          // A pointer to our settings
    private Socket connSock;
    private String fromMU;              // a string to hold output from the MU*
    private MUDBufferedReader inStream; // input to us from the MU*
    private DataOutputStream outStream; // for sending info to the MU*
    private Vector heldResponse;        // A vector to hold any text incase we are "paused"
    private int minLineCount = 0; // the number of minimised lines
    private CHandler connHandler;
    
    private boolean useSwing = false;
    private String logTitle;    // An auto-generated file name for logs specific to this session
    private boolean logging = false;    // indicate if auto-logging should be occurring
    private boolean autoConnect = false;
    private String connectionStr = "";
    
    private java.io.FileOutputStream logFile;	// The outputstream for our log file
    private java.io.PrintWriter logWriter;      // The Printwriter for our log file
    
    private static final boolean DEBUG = false;
    
    
    
    /** The constructor.  Behold!! */
    public MuSocket() {
        // this.settings = settings;
        settings = JMConfig.getInstance();
        
        if (settings.getJMboolean(JMConfig.USESWING)) {
            useSwing = true;
        }
        
        // Set up a new text area
        if (useSwing) {
            
            java.awt.Color[] newPal = (java.awt.Color[])settings.getJMObject(JMConfig.CUSTOMPALETTE);
            mainSText = new JMSwingText();
            mainSText.addMouseListener(this);
            mainSText.setForeground(settings.getJMColor(settings.FOREGROUNDCOLOUR));
            mainSText.setBackground(settings.getJMColor(settings.BACKGROUNDCOLOUR));
            mainSText.setFont(settings.getJMFont(settings.FONTFACE));
            mainSText.setPaintBackground(settings.getJMboolean(JMConfig.BGPAINT));
            mainSText.setEditable(false);
            // mainSText.addKeyListener(settings.getMainWindowVariable());
            mainSText.addKeyListener(MuckMain.getInstance());
            mainSText.setAntiAliasing(settings.getJMboolean(JMConfig.ANTIALIAS));
            mainSText.setBoldNotBright(settings.getJMboolean(JMConfig.LOWCOLOUR));
            
            try {
                mainSText.setPalette(newPal);
            } catch(Exception exc) {
                if (DEBUG) {
                    System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Error_setting_custom_palette_from_MuSocket."));
                }
            }
            
        } else {
            mainText = new JMText(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString(""), 50, 80, JMText.SCROLLBARS_VERTICAL_ONLY);
            
            // Set the attributes of our text area
            
            mainText.setEditable(false);
            mainText.setDoubleBuffer(settings.getJMboolean(JMConfig.DOUBLEBUFFER));
            mainText.addMouseListener(this);
            mainText.setForeground(settings.getJMColor(settings.FOREGROUNDCOLOUR));
            mainText.setBackground(settings.getJMColor(settings.BACKGROUNDCOLOUR));
            mainText.setFont(settings.getJMFont(settings.FONTFACE));
            
        }
        
        
        
        heldResponse = new Vector(0, 1);
        
        connHandler = settings.getConnectionHandler();
        
    }
    
    
    
    // Check for mouse actions
    
    
    
    /**
     *
     *
     *
     * @param mEvent
     *
     */
    public void mousePressed(final MouseEvent mEvent) {
        setPaused(true);
    }
    
    
    
    /**
     *
     *
     *
     * @param mEvent
     *
     */
    
    public void mouseReleased(final MouseEvent mEvent) {
        
    }
    
    
    
    /**
     *
     *
     *
     * @param mEvent
     *
     */
    
    public void mouseClicked(final MouseEvent mEvent) {
        
        // Determine number of mouse clicks
        final int count = mEvent.getClickCount();
        
        
        
        if (count >= 2) {
            // Call outside routines (plug-ins) to handle the selected text
            mEvent.consume();
            spoolText();
            
            // grab the 'selected' URL
            String tempText;
            
            
            if (useSwing) {
                // We're going to have to do some manual manipulation of the text, as the
                // double-click on the JTextArea doesn't grab the whole token by itself.
                if (DEBUG) {
                    System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.mouseClicked_trying_to_get_information_from_mainSText()_") + mainSText);
                }
                // tempText = (mainSText.getSelectedText()).trim();
                // tempText = fullURL(mainText.getSelected)
                tempText = fullURL();
                
            } else {
                tempText = (mainText.getSelectedText()).trim();
            }
            
            final StringBuffer tentativeURL = new StringBuffer(tempText);
            
            // StringBuffer tentativeURL = new StringBuffer((mainText.getSelectedText()).trim());
            
            if (DEBUG) {
                
                System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.mouseClicked()_using_") + tentativeURL + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("_for_external_program."));
                
            }
            
            
            
            // Fix this XXX
            // The frame is sent to anchor any potential dialogues
            // send the 'URL' to launch the correct program
//            final ExternalProgs extp = new ExternalProgs(settings.getJMFrame(JMConfig.MUCKMAINFRAME));
//
//            extp.launchProgram(settings.getJMFrame(JMConfig.MUCKMAINFRAME), tentativeURL);
            
//            final anecho.extranet.Bowsah webBrowser = new anecho.extranet.Bowsah(tentativeURL.toString());
//            webBrowser.displayURL();
            final BrowserWrapper wrapper = BrowserWrapper.getInstance();
            wrapper.showURL(tentativeURL.toString());
            
            // As a quick fix we'll copy this to the Clipboard instead.
            if (useSwing) {
                // java.awt.datatransfer.Clipboard clip = mainSText.getToolkit().getSystemClipboard();
                mainSText.copy();
            } else {
                final java.awt.datatransfer.Clipboard clip = mainText.getToolkit().getSystemClipboard();
                
                try {
                    final java.awt.datatransfer.StringSelection selection = new java.awt.datatransfer.StringSelection(tentativeURL.toString());
                    clip.setContents(selection, selection);
                } catch (Exception clipOopse) {
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket_exception_copying_to_Clipboard.") + clipOopse);
                }
                
                // Deselect the text
                
                mainText.select(mainText.getSelectionEnd(), mainText.getSelectionEnd());
                
                if (DEBUG) {
                    System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket:_Deselect_text?"));
                }
                
            }
            
        } else {
            
            // A single click signals a pause
            // This will pause the output window,
            // the incoming lines held in queue
            setPaused(true);
            
        }
        
    }
    
    
    
    /**
     *
     *
     *
     * @param mEvent
     *
     */
    
    public void mouseEntered(final MouseEvent mEvent) {
        
    }
    
    
    
    /**
     *
     *
     *
     * @param mEvent
     *
     */
    
    public void mouseExited(final MouseEvent mEvent) {
        
    }
    
    
    
    /**
     *
     * The name of our connection
     *
     * @param newName
     *
     */
    
    public synchronized void setTitle(final String newName) {
        
        if (DEBUG) {
            
            System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.setTitle_setting_muckName_to:_") + newName);
            
        }
        
        muckName = newName;
        
        setFrameTitles();
        
        if (DEBUG) {
            
            System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.setTitle_complete."));
            
        }
        
        
        
    }
    
    
    
    /**
     *
     * getTitle does a small bit of book-keeping, returning a title based
     *
     * on whether we're connected to the MU* or not.  If we're not connected
     *
     * then we'll return &quot;JamochaMUD" instead of the name
     *
     * @return
     *
     */
    
    public synchronized String getTitle() {
        
        String retStr;
        
        
        
        if (!connected) {
            
            retStr = java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Not_Connected:_") + address;
            
        } else {
            
            
            
            if (muckName == null) {
                
                if (DEBUG) {
                    
                    System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.getTitle_->_current_muckName_is_null_and_will_be_set_to_") + address);
                    
                }
                
                muckName = address;
                
            }
            
            retStr = java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("JamochaMUD_(") + muckName + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString(")");
            
        }
        
        
        
        return retStr;
        
    }
    
    
    
    /**
     *
     * This returns the true name of the MU*, regardless of our connection
     *
     * @return
     *
     */
    
    public String getMUName() {
        
        return muckName;
        
    }
    
    
    
    /** Call this to reset the frame's title to the &quot;standard" title.
     *
     * This is usually just the location */
    
    public synchronized void resetTitle() {
        
        setFrameTitles();
        
    }
    
    
    
    /**
     *
     * The Address of our connection
     *
     * @param address
     *
     */
    
    public void setAddress(final String address) {
        
        this.address = address;
        
    }
    
    
    
    /**
     *
     *
     *
     * @return
     *
     */
    
    public String getAddress() {
        
        return address;
        
    }
    
    
    
    /**
     *
     * The port our connection will connect to
     *
     * @param port
     *
     */
    
    public void setPort(final int port) {
        
        this.port = port;
        
    }
    
    
    
    /**
     *
     *
     *
     * @return
     *
     */
    
    public int getPort() {
        
        return port;
        
    }
    
    
    
    /**
     *
     * @param ssl
     */
    public void setSSL(boolean ssl) {
        
        if (DEBUG) {
            
            System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Our_address_has_been_set_to_a_secure_socket."));
            
        }
        
        this.ssl = ssl;
        
    }
    
    
    
    /**
     *
     * @return
     */
    public boolean getSSL() {
        
        return ssl;
        
    }
    
    
    
    /**
     *
     * The timestamp of our connection, for internal use mostly
     *
     * @param time
     *
     */
    
    public void setTimeStamp(final long time) {
        
        this.timeStamp = time;
        
    }
    
    
    
    /**
     *
     *
     *
     * @return
     *
     */
    
    public synchronized long getTimeStamp() {
        
        return timeStamp;
        
    }
    
    
    
    /** This is the main method for our thread.  Veryvery important!
     *  Basically, it'll be a big loop that only gets broken when our
     * connection disappears... either from the socket's end or the
     * user's end */
    
    public void run() {
        
        final String tempAddy = this.getAddress();
        
        // Give some visual notification that we're attempting a connection
        setVisuals(tempAddy);
        
        // Write the location to the main window's title, as well
        
        // Try to establish a connection
        final int port = this.getPort();
        InetAddress serverAddy;
        
        // We'll give sockPort and sockHost initial values to satisfy later conditions
        int sockPort = 0;
        String sockHost = java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("");
        final boolean socks = settings.getJMboolean(JMConfig.PROXY);
        
        if (socks) {
            sockHost = settings.getJMString(JMConfig.PROXYHOST);
            sockPort = settings.getJMint(JMConfig.PROXYPORT);
        }
        
        try {
            serverAddy = InetAddress.getByName(tempAddy);
            if (socks) {
                if (DEBUG) {
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Attempt_to_use_Socks5socket."));
                }
                connSock = (Socket)new Socks5socket(tempAddy, port, sockHost, sockPort);
                if (DEBUG) {
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Socks5socket_successfully_connected"));
                }
                
                // outStream = new DataOutputStream(connSock.getOutputStream());
                
            } else {
                if (getSSL()) {
                    // We need to make a secure connection
                    // Create a new TrustManager that will blindly trust certificates.
                    // Most MU*'s have self-signed certs, and would otherwise never let us connect
                    
                    javax.net.ssl.TrustManager[] trustAllCerts = new javax.net.ssl.TrustManager[]{new anecho.extranet.NoTrustManager()};
                    
                    javax.net.ssl.SSLContext sc = javax.net.ssl.SSLContext.getInstance(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("SSL"));
                    sc.init(null, trustAllCerts, new java.security.SecureRandom());
                    
                    javax.net.ssl.SSLSocketFactory sslFact = sc.getSocketFactory();
                    javax.net.ssl.SSLSocket sslSock = (javax.net.ssl.SSLSocket)sslFact.createSocket(serverAddy, port);
                    
                    sslSock.startHandshake();
                    
                    outStream = new DataOutputStream(sslSock.getOutputStream());
                    inStream = new MUDBufferedReader(sslSock.getInputStream(), outStream);
                    
                    // We do this so that the SSL socket may be closed in "closeSocket()" method
                    connSock = (Socket)sslSock;
                    
                } else {
                    connSock = new Socket(serverAddy, port);
                    // outStream = new DataOutputStream(connSock.getOutputStream());
                    // inStream = new MUDBufferedReader(connSock.getInputStream(), outStream);
                    
                }
                
            }
            
            if (outStream == null) {
                outStream = new DataOutputStream(connSock.getOutputStream());
            }
            
            if (inStream == null) {
                inStream = new MUDBufferedReader(connSock.getInputStream(), outStream);
            }
            
            // outStream = new DataOutputStream(connSock.getOutputStream());
            
        } catch (NoRouteToHostException oops) {
            // No route to host, or operation timed out
            disconnectMenu();
            
            final String tempString = oops + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("");
            if (tempString.endsWith(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("unreachable"))) {
                // Host unreachable
                write(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("noRouteToHostException") + '\n');
            } else {
                write(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("operationTimedOutException") + '\n');
            }
        } catch (UnknownHostException oops) {
            disconnectMenu();
            write(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("unknownHostException") + '\n');
        } catch (ConnectException oops) {
            disconnectMenu();
            write(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("connectException") + '\n');
        } catch (Exception oops) {
            disconnectMenu();
            
            if (DEBUG) {
                System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("From_Net,_no_socket_") + oops);
                oops.printStackTrace();
            }
            
            write(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("exception") + '\n');
            
            // Fix this XXX
            // closeConnection();
            
        }
        
        // Create a MUDBufferedReader for our input stream from the MU*
        
        try {
//            if (getSSL()) {
//                // inStream = new MUDBufferedReader(((javax.net.ssl.SSLSocket)connSock).getInputStream(), outStream);
//            } else {
//                inStream = new MUDBufferedReader(connSock.getInputStream(), outStream);
//            }
            
            inStream.addTelnetEventListener(settings.getDataInVariable());
            inStream.addTelnetEventListener(this);
            
        } catch (Exception except) {
            
            if (DEBUG) {
                System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.run_->_No_inStream") + except);
                except.printStackTrace();
            }
            
            // mainText.append("---> " + e + " <---" + '\n');
            write(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("--->_") + except + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("_<---") + '\n');
            
            closeCleanUp();
            
            return;
            
        }
        
        
        
        // Okay, we're connected.  Do our little set-up stuff
        // Some specialised MU*'s require a form of authentication immediately
        // after connection.  This could be done here by adding something like:
        // sendText("foo");
        // Read from the socket until we get tired and fall down
        
        connected = true; // This notifies the program of true connection
        
        // Change the titles of our frames to show we're connected
        setFrameTitles();
        
        // Update the main window menus to reflect our connected state
        connectMenu();
        
        // Add auto-connection information here
        // Fix Me XXX!!
        if (autoConnect) {
            if (DEBUG) {
                System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.run()_attempting_autoConnect."));
            }
            
            if (!connectionStr.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString(""))) {
                sendText(connectionStr);
            }
            
        }
        
        try {
            while (connected) {
                // Commented out 2005-09-24 to move exception handling outside of while loop
                fromMU = inStream.readLine();
                fromMU = EnumPlugIns.callPlugin(fromMU, java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("output"), this);
                
                // Send the current line to our Text-field
                if (!paused) {
                    
                    // Check for any URLs
                    fromMU = checkURL(fromMU);
                    
                    write(fromMU);
                    
                    if (DEBUG) {
                        System.err.println(getMUName() + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("_is_listed_as_active?_") + isActiveMU());
                    }
                    // We'll notify the main window if needed
                    if (!isActiveMU()) {
                        if (DEBUG) {
                            System.err.println(getMUName() + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("_is_an_inactive_mu.__Our_activity_level_is_") + activity);
                        }
                        if (activity < 2) {
                            // Append an activity notice to the active window
                            // connHandler.getActiveMUDText().append(resReader("activityOn") + " " + getMUName() + '\n');
                            if (DEBUG ) {
                                System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Activity_on") + getMUName() + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString(".__Change_tab_colour_requested"));
                            }
                            // connHandler.getActiveMUDText().append(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("activityOn") + getMUName() + '\n');
                            if (useSwing) {
                                anecho.gui.JMFancyTabbedPane ftp = (anecho.gui.JMFancyTabbedPane)settings.getJMObject(JMConfig.TEXTPANEL);
                                ftp.flashTab(mainSText, true);
                            } else {
                                connHandler.getActiveMUDText().append(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("activityOn") + getMUName() + '\n');
                            }
                            
                            activity++;
                        }
                        
                    } else {
                        
                        if (DEBUG) {
                            System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Resetting_activity_count_on_") + getMUName());
                        }
                        activity = 0;
                        // System.out.println("Reset activity.");
                        
                    }
                    
                    
                    
                    // Check to see if the main frame is minimised.  If so, we add
                    
                    // the new text to the title bar
                    
                    // if (settings.isMainWindowIconified()) {
                    
                    if (settings.getJMboolean(JMConfig.MAINWINDOWICONIFIED)) {
                        
                        addTitleText(fromMU);
                        
                    } else {
                        
                        minLineCount = 0;
                        
                    }
                    
                    
                    
                } else {
                    
                    // System.out.println("paused: " + fromMU);
                    
                    heldResponse.addElement(fromMU);
                    
                }
                
            }
            
        } catch (Exception exc) {
            
            if (DEBUG) {
                
                System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.run():_We've_fallen_out_of_our_while_loop.") + exc);
                
                exc.printStackTrace();
                
            }
            
            // Check to see if we somehow got disconnected.  (It can happen!)
            
            // sets out menu to the correct state as well as closes the socket
            
            // mainText.append("---> " + resReader("connectionClosed") + " <---" + '\n');
            
            // write("---> " + resReader("connectionClosed") + " <---" + '\n');
            
            write("---> " + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("connectionClosed") + "<---\n");
            
            closeSocket();
            
            connected = false;
            
            disconnectMenu();
            
            
            
        }
        
    }
    
    
    
//    private static String resReader(final String itemTarget) {
    
//        final ResReader reader = new ResReader();
    
//        return reader.langString(JMConfig.BUNDLEBASE, itemTarget);
    
//    }
    
    
    
    // Return our text area to who-ever wants to know! */
    
    /**
     *
     *
     *
     * @return
     *
     */
    
    public synchronized JMText getTextWindow() {
        
        return mainText;
        
    }
    
    
    
    /* Return the swing version of our text window */
    
    /**
     *
     *
     *
     * @return
     *
     */
    
    public synchronized JMSwingText getSwingTextWindow() {
        
        return mainSText;
        
    }
    
    
    
    /**
     *
     * Send text out to our MU*
     *
     * @param send
     *
     */
    
    public void sendText(final String send) {
        
        try {
            
            // if (settings.getUseUnicode()) {
            
            if (settings.getJMboolean(JMConfig.USEUNICODE)) {
                
                // Send the string as Unicode
                
                // outStream.writeChars(send + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("\r\n"));
                if (outStream != null) {
                    outStream.writeChars(send + "\r\n");
                }
                
            } else {
                
                // Send the string as regular ANSI text
                if (outStream != null) {
                    outStream.writeBytes(send + "\r\n");
                }
                
            }
            
        } catch (Exception except) {
            
            // Most likely we aren't connected to the MU*
            if (DEBUG) {
                System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.sendText_exception:_") + except);
                except.printStackTrace();
            }
        }
        
    }
    
    
    
    /** This will set up some of the initial indicators
     *
     * that we're trying to make a connection */
    
    private void setVisuals(final String title) {
        
        // mainText.append(resReader("attemptingConnectionTo") + " " + title + "..." + '\n');
        
        // write(resReader("attemptingConnectionTo") + " " + title + "..." + '\n');
        
        // write(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("attemptingConnectionTo") + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("_") + title + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("...\n"));
        write(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("attemptingConnectionTo") + " " + title + "...\n");
        
        
        
    }
    
    
    
    private void setFrameTitles() {
        
        // We should only be able to change the frame titles it this
        
        // is the active MU*
        
        if (connHandler.getActiveMUHandle() == this) {
            
            final MuckMain target = MuckMain.getInstance();
            
            target.setWindowTitle();
            
            
            
            final DataIn inWindow = settings.getDataInVariable();
            
            inWindow.setWindowTitle();
            
        }
        
    }
    
    
    
    /**
     *
     * Return the status of our connection:<br>
     *
     * @return <CODE>true</CODE> - Connected
     *
     * <CODE>false</CODE> - Not connected
     *
     */
    
    public boolean isConnectionActive() {
        
        if (DEBUG) {
            
            System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.isConnectionActive()_returns:_") + connected);
            
        }
        
        return connected;
        
    }
    
    
    
    /** Set the proper states of our windows, etc.
     *
     * 'cause, woe is us, we are disconnected.  Most often
     *
     * this would be called internally by our class
     *
     */
    
    private void disconnectMenu() {
        
        // This is where we actually disconnect the socket.
        
        // Is there a nicer way of saying good-bye?
        
        
        
        // Do we really need to close the socket?!  We should get here because the
        
        // socket is already closed!  Check this out! XXX
        
        // This guarantees that we've cleaned up after ourselves, too
        
        
        
        // Change the connection notice on the main window if applicable
        
        // final MuckMain window = settings.getMainWindowVariable();
        
        final MuckMain window = MuckMain.getInstance();
        
        window.checkDisconnectMenu();
        
        
        
    }
    
    
    
    /** Allow other classes to close our socket */
    
    public void closeSocket() {
        
        /*
         
        if (DEBUG) {
         
            StringWriter sw = new StringWriter();
         
            new Throwable().printStackTrace(
         
                                            new PrintWriter( sw )
         
                                           );
         
            String callStack = sw.toString();
         
         
         
            System.out.println("Stack: " + callStack);
         
        }
         
         */
        
        
        
        try {
            
            if (connSock != null) {
                
                connSock.close();
                
            }
            
        } catch (Exception closeError) {
            
            // The only reason we should get here is if the socket was never opened
            
            // in the first place.  This can happen if we tried to open a bad address
            
            if (DEBUG) {
                
                System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket_hit_an_error_(.disconnectMenu):_") + closeError);
                
            }
            
        }
        
        
        
        // Change the titles of the frames if applicable
        
        resetTitle();
        
        
        
        // return;
        
    }
    
    
    
    /** Set the proper states of our windows as
     *
     * we've probably just connected
     *
     */
    
    private void connectMenu() {
        
        // final MuckMain main = settings.getMainWindowVariable();
        
        final MuckMain main = MuckMain.getInstance();
        
        main.setConnectionMenu();
        
    }
    
    
    
    /**
     *
     * Set the paused status of our text area
     *
     * @param option
     *
     */
    
    public void setPaused(final boolean option) {
        
        paused = option;
        
        
        
        if (option) {
            
            setFrameTitles();
            
        } else {
            
            spoolText();
            
        }
        
    }
    
    
    
    /**
     *
     * Return the fact that our connection is either
     *
     * paused - <pre>true</pre><br>
     *
     * or not paused - <pre>false</pre><br>
     *
     * @return
     *
     */
    
    public synchronized boolean isPaused() {
        
        return paused;
        
    }
    
    
    
    /**
     *
     * Write any "held" text out to the user's display.
     *
     */
    
    public synchronized void spoolText() {
        
        // We'll reset this variable, just in case
        
        paused = false;
        
        
        
        // System.out.println("Spooltext, " + heldResponse.size() + " entries.");
        
        
        
        // Reset our frame titles
        
        resetTitle();
        
        
        
        // Do a little bounds checking first
        
        if (heldResponse.size() < 1) {
            
            /* System.out.println("MuSocket: spoolText, nothing to spool."); */
            
            return;
            
        }
        
        
        
        // System.out.println("MuSocket.spoolText: we have " + heldResponse.size() + " elements.");
        
        while (heldResponse.size() > 0) {
            
            // System.out.println("MuSocket.spoolText: " + heldResponse.elementAt(0).toString());
            
            // mainText.append(heldResponse.elementAt(0).toString());
            
            write(heldResponse.elementAt(0).toString());
            
            if (DEBUG) {
                
                System.out.println(heldResponse.elementAt(0).toString());
                
            }
            
            
            
            heldResponse.removeElementAt(0);
            
        }
        
    }
    
    
    
    /** Trim down the string we've received from the MU*, add
     *
     * the minimised line count, and the set the new title */
    
    private synchronized void addTitleText(final String newText) {
        
        minLineCount++;
        
        // System.out.println("MuSocket.addTitleText: " + minLineCount + " " + newText);
        
        
        
        final String str = stripEscapes(newText);
        
        
        
        // Trim the str down to a managable length
        
        String trimString;
        
        if (str.length() > 80) {
            
            trimString = str.substring(0, 80) + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("...");
            
        } else {
            
            trimString = str;
            
        }
        
        
        
        // final MuckMain target = settings.getMainWindowVariable();
        
        final MuckMain target = MuckMain.getInstance();
        
        // target.setWindowTitle("(" + minLineCount + ") " + str);
        
        target.setWindowTitle(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("(") + minLineCount + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString(")_") + trimString);
        
        
        
    }
    
    
    
    /** This method returns the value of a string
     *
     * once it has had any escape characters stripped
     *
     * from it.
     *
     */
    
    private String stripEscapes(final String token) {
        
        final StringBuffer workString = new StringBuffer(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString(""));
        
        boolean loop = true;
        
        int start = 0;
        
        int end = 0;
        
        
        
        while (loop) {
            
            end = token.indexOf('\u001b', start);
            
            
            
            if (end < start) {
                
                // There are no escapes left
                
                workString.append(token.substring(start));
                
                loop = false;
                
                break;
                
            }
            
            
            
            if (end > 0) {
                
                workString.append(token.substring(start, end));
                
            }
            
            
            
            // Now set the new 'start'
            
            start = token.indexOf('m', end) + 1;
            
            if (start <= end) {
                
                loop =false;
                
                System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Not_a_proper_ANSI_escape"));
                
                break;
                
            }
            
        }
        
        
        
        return workString.toString();
        
    }
    
    
    
    private void closeCleanUp() {
        
        if (DEBUG) {
            
            System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.closeCleanUp()_calling_closeSocket()"));
            
        }
        
        closeSocket();
        
        connected = false;
        
        if (DEBUG) {
            
            System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.closeCleanUp()_calling_disconnectMenu()"));
            
        }
        
        disconnectMenu();
        
        if (DEBUG) {
            
            System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.closeCleanUp()_completed."));
            
        }
        
    }
    
    
    
    /** Returns <pre>true</pre> if this connection is the currently active
     *
     * (visible to the user) MU* connection.  <pre>False</pre> is returned
     *
     * otherwise.
     *
     */
    
    private boolean isActiveMU() {
        
        return activeStatus;
        
    }
    
    
    
    /**
     *
     * Indicate that this MU active or inactive
     *
     * @param status
     *
     */
    
    public void setActiveMU(final boolean status) {
        
        activeStatus = status;
        
        if (DEBUG) {
            System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("active_state:_") + status + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("_for_") + getTitle());
        }
        
        // If this MU* is set active then we can reset our activity counter
        if (status) {
            if (DEBUG) {
                System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Setting_out_activity_monitor_to_0"));
            }
            activity = 0;
            if (useSwing) {
                // reset tab colour if appropriate Fix me XXX
                anecho.gui.JMFancyTabbedPane ftp = (anecho.gui.JMFancyTabbedPane)settings.getJMObject(JMConfig.TEXTPANEL);
                ftp.flashTab(mainSText, false);
            }
            
        }
    }
    
    
    public void validate() {
        
        System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.validate_called."));
        
        if (useSwing) {
            
            mainSText.validate();
            
        } else {
            
            mainText.validate();
            
        }
        
    }
    
    
    
    /**
     *
     * Return the state of our "echo"
     *
     * @return
     *
     */
    
    public boolean getEchoState() {
        
        return echo;
        
    }
    
    
    
    private void setEchoState(final boolean state) {
        
        echo = state;
        
        System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket_echo_state:_") + echo);
        
    }
    
    
    
    /** Check the string for anything understood as a URL and underline it */
    
    private String checkURL(final String input) {
        
        String checkChunk, lCheckChunk;
        
        final StringBuffer incoming = new StringBuffer(input);
        
        
        
        final java.util.StringTokenizer tokens = new java.util.StringTokenizer(input);
        
        
        
        while (tokens.hasMoreTokens()) {
            
            checkChunk = tokens.nextToken();
            
            lCheckChunk = checkChunk.toLowerCase();
            
            if (lCheckChunk.indexOf(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("www.")) >= 0 || lCheckChunk.indexOf(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("http://")) >= 0) {
                
                int match;
                
                match = (incoming.toString()).indexOf(checkChunk);
                
                
                
                // Fix ME XXX!!!
                
                // Using Swing we may want to change this to a "label" and insert it as a
                
                // component into our JMSwingText component
                
                
                
                // Added back-to-front so we don't have to do extra calculations
                
                if (!useSwing) {
                    
                    // if (lCheckChunk.indexOf("http.") < 0) {
                    
                    // Add the http:// so that our link conforms to the URL standard
                    
                    // incoming.insert(match, "http://");
                    
                    // }
                    
                    // incoming.insert(match + checkChunk.length(), "</a>");
                    
                    // incoming.insert(match, "<a href=\"" + lCheckChunk + "\">");
                    
                    // } else {
                    
                    incoming.insert(match + checkChunk.length(), java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("\u001b[24m"));
                    
                    incoming.insert(match, java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("\u001b[4m"));
                    
                }
                
                
                
            }
            
        }
        
        
        
        return incoming.toString();
        
    }
    
    
    
    /** Write out given text to our text component */
    
    private synchronized void write(final String output) {
        
        if (useSwing) {
            
            mainSText.append(output);
            
        } else {
            
            mainText.append(output);
            
        }
        
        
        
        // if logging is enabled, this should be written to the log as well
        
        if (logging) {
            
            writeLog(output);
            
        }
        
    }
    
    
    
    /*
     
    public void setDoubleBuffer(boolean state) {
     
        mainText.setDoubleBuffer(state);
     
    }
     
     
     
    public void setFont(Font fontStyle) {
     
        mainText.setFont(fontStyle);
     
    }
     
     
     
    public void setForeground(Color fgColor) {
     
        mainText.setForeground(fgColor);
     
    }
     
     
     
    public void setBackground(Color bgColor) {
     
        mainText.setBackground(bgColor);
     
    }
     
     */
    
    
    
    /**
     *
     * Custom telnet message receiver for our TelnetEventListener
     *
     * @param event
     *
     */
    
    public void telnetMessageReceived(final TelnetEvent event) {
        
        
        
        if (DEBUG) {
            
            System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.telnetMessageReceived_has_received:_") + event);
            
            System.out.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("With_our_message_being:_") + event.getMessage());
            
        }
        
        
        
        final String message = event.getMessage();
        
        if (message.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("IAC_DO_TELOPT_ECHO"))) {
            
            // Turn off the local echo
            
            setEchoState(false);
            
        }
        
        
        
        if (message.equals(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("IAC_DONT_TELOPT_ECHO"))) {
            
            // Turn the local echo back on.
            
            setEchoState(true);
            
        }
        
    }
    
    
    
    /**
     *
     * Enable or disable auto-logging
     *
     * @param status
     *
     */
    
    public void setLogging(final boolean status) {
        
        logging = status;
        
        if (DEBUG) {
            
            System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Logging_has_been_set_to_") + status);
            
        }
        
        
        
        if (!status && logFile != null) {
            
            // Logging has been disabled.  Close our file handle
            
            try {
                
                logWriter.flush();
                
                logWriter.close();
                
            } catch (Exception exc) {
                
                if (DEBUG) {
                    
                    System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Exception_trying_to_flush_and_close_logWriter_in_MuSocket.setLogging"));
                    
                }
                
            }
            
            
            
            logFile = null;
            
            
            
        }
        
    }
    
    
    
    /**
     *
     * Return our auto-logging status
     *
     * @return
     *
     */
    
    public boolean isLogging() {
        
        return logging;
        
    }
    
    
    
    /** Return a string that should be used as a file name for logging this session
     *
     *  If a title does not yet exist, we will create one
     *
     *  Title format: MU* address + timestamp + .txt
     *
     */
    
    private String fileTitle() {
        
        String workingTitle;
        
        
        
        if (logTitle == null) {
            
            // Assemble a date string that is human readable from our timestamp
            
            final java.util.Date workDate = new java.util.Date(timeStamp);
            
            final java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("yy-MM-dd_H-mm-ss"));
            
            
            
            // Create the title for our file and assign it to the logTitle variable
            
            final StringBuffer tempStr = new StringBuffer();
            
            tempStr.append(muckName);
            
            tempStr.append(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("_"));
            
            tempStr.append(formatter.format(workDate));
            
            tempStr.append(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString(".txt"));
            
            
            
            workingTitle = tempStr.toString();
            
            
            
            // Set up our logTitle
            
            logTitle = workingTitle;
            
            
            
            if (DEBUG) {
                
                System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Our_new_logTitle_is_") + logTitle);
                
            }
            
            
            
            
            
        } else {
            
            workingTitle = logTitle;
            
        }
        
        
        
        return workingTitle;
        
    }
    
    
    
    /** Write our output to our log file
     *
     */
    
    private void writeLog(final String output) {
        
        if (logFile == null) {
            
            // Setup our variables for writing
            
            // We'll keep the append option to true, as we can't tell if the
            
            // user stopped logging and then perhaps started again.
            
            // String logPath = settings.getJMString(JMConfig.LOGPATH) + fileTitle();
            
            
            
            final String logPath = settings.getJMString(JMConfig.USERDIRECTORY) + java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("logs");
            
            final String totalLogFile = logPath + settings.getJMString(JMConfig.PATHSEPARATOR) + fileTitle();
            
            
            
            if (DEBUG) {
                
                System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Our_current_logpath_is:_") + logPath);
                
                System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Our_total_logfile_is:_") + totalLogFile);
                
            }
            
            
            
            try {
                
                // java.io.File logFileName = new java.io.File(fileTitle());
                
                if (!new java.io.File(logPath).exists()) {
                    
                    if (DEBUG) {
                        
                        System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Attempting_to_create_directory_for_logs"));
                        
                    }
                    
                    new java.io.File(logPath).mkdirs();
                    
                    if (DEBUG) {
                        
                        System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("Successfully_created_log_directory."));
                        
                    }
                    
                }
                
                
                
                logFile = new java.io.FileOutputStream(totalLogFile, true);
                
                logWriter = new java.io.PrintWriter(logFile, true);
                
            } catch (Exception exc) {
                
                if (DEBUG) {
                    
                    System.err.println(java.util.ResourceBundle.getBundle("anecho/JamochaMUD/JamochaMUDBundle").getString("MuSocket.writeLog:_") + exc);
                    
                }
                
            }
            
            
            
        }
        
        
        
        final String cleanOut = anecho.gui.TextUtils.stripEscapes(output, false);
        
        // logWriter.print(output);
        
        logWriter.print(cleanOut);
        
        logWriter.flush();
        
    }
    
    /**
     *
     * @param status
     */
    public void setAutoConnect(final boolean status) {
        autoConnect = status;
    }
    
    /**
     *
     * @param connStr
     */
    public void setConnectString(final String connStr) {
        connectionStr = connStr;
    }
    
    private String fullURL() {
        // Set our return string to the selected text just in case
        String retStr = mainSText.getSelectedText();
        
        int start = mainSText.getSelectionStart();
        int end = mainSText.getSelectionEnd();
        
        if (DEBUG) {
            System.err.println("MuSocket.fullURL original indexes (start, end)" + start + ", " + end);
            System.err.println("MuSocket.fullURL original selection: " + retStr);
        }
        final String workStr = mainSText.getText();
        
        // Change the end value to the position of the next white-space.
        // This may end up getting some punctuation, but we'll take care
        // of that after.
        int testEnd = workStr.indexOf(' ', end);
        final int breakEnd = workStr.indexOf('\n', end);
        
        // Avoid wrapping on to a new line if there is a line-break
        if (breakEnd < testEnd) {
            testEnd = breakEnd;
        }
        
        if (testEnd > end) {
            end = testEnd;
        }
        
        if (testEnd < 0) {
            end = workStr.length();
        }
        
        // Change the start value to the position of white-space (or beginning
        // of the text.  We can't search backwards through the text as we cannot
        // always be certain if there are illegal characters.
        int testStart = workStr.lastIndexOf("http://", start);
               
        // Add 1 to avoid capturing the space as well
        final int spaceStart = workStr.lastIndexOf(' ', start) + 1;
        
        // This helps to determine the beginning of the URL if there is already
        // another URL on the same line.
        if (spaceStart > testStart) {
            testStart = spaceStart;
        }
        
        
        if (testStart < start && testStart > -1) {
            start = testStart;
        }
        
        retStr = workStr.substring(start, end);
        
        if (DEBUG) {
            System.err.println("MuSocket.fullURL() returns a string of " + retStr);
            System.err.println("MuSocket.fullURL() our new indexes are (start, end): " + start + ", " + end);
        }

        // Set the new selection based on what we've worked up here.
        mainSText.setSelectionEnd(end);
        mainSText.setSelectionStart(start);
        
        return retStr;
        
    }
}

