/*
 * TextUtils.java
 *
 * Created on October 28, 2005, 10:33 PM
 *
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package anecho.gui;

/**
 * This class contains methods to do some common string requirements
 * such as stripping escape characters.
 * @author Jeff Robinson
 */
public class TextUtils {
    
    private static final char BELL = '7';
    private static final boolean DEBUG = false;
    
    /** Creates a new instance of TextUtils */
    private TextUtils() {
    }
    
    /**
     * This method returns the value of a string
     * once it has had any escape characters stripped
     * from it.
     * @param token The string that will be analysed for escape characters.
     * @param useBell <CODE>true</CODE> - trigger system "bell" if a bell sequence is stripped.
     * <CODE>false</CODE> - do not trigger the system "bell" if a bell sequence is stripped.
     * @return Returns the initial string with escapes removed.
     */
    public static synchronized String stripEscapes(final String token, final boolean useBell) {
        final StringBuffer workString = new StringBuffer(java.util.ResourceBundle.getBundle("anecho/gui/guiBundle").getString(""));
        boolean loop = true;
        int start = 0;
        int end = 0;
        final int tLen = token.length();
        
        do {
            // Look for the next escape character
            end = token.indexOf('\u001b', start);
            
            // if (end < start && start < 0) {
            if (start < 0) {
                // There are no escapes left
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/gui/guiBundle").getString("We_break_to_avoid_an_index_error."));
                    break;
            }
            
            if (end < start) {
                workString.append(token.substring(start));
                loop = false;
                break;
            }
            
            // if (end > 0 && start >= 0) {
            if (end > 0) {
                workString.append(token.substring(start, end));
            }
            
            // Now set the new 'start'
            start = token.indexOf('m', end) + 1;
            
            if (start <= end) {
                loop = false;
                if (start < 0) {
                    System.out.println(java.util.ResourceBundle.getBundle("anecho/gui/guiBundle").getString("We_don't_have_an_'m'_to_end_our_ANSI_escape.__We_may_not_have_received_it_yet."));
                } else {
                    // Check for BELL character (this is probably an ugly place to stick this Fix Me XXX)
                    if (token.charAt(1) == BELL && useBell) {
                        java.awt.Toolkit.getDefaultToolkit().beep();
                        if (DEBUG) {
                            System.out.println(java.util.ResourceBundle.getBundle("anecho/gui/guiBundle").getString("Start:_") + start + java.util.ResourceBundle.getBundle("anecho/gui/guiBundle").getString("_end:_") + end);
                        }
                        start = 2;
                        if (end == 0 && tLen > 2) {
                            // There are no more escapes, so we'll just add the remainder of the string
                            workString.append(token.substring(2));
                        }
                        
                        break;
                    } else {
                        if (DEBUG) {
                            System.out.println(java.util.ResourceBundle.getBundle("anecho/gui/guiBundle").getString("Not_a_proper_ANSI_escape"));
                            
                            for (int i = 0; i < tLen; i++) {
                                System.out.print(java.util.ResourceBundle.getBundle("anecho/gui/guiBundle").getString("*") + token.charAt(i) + java.util.ResourceBundle.getBundle("anecho/gui/guiBundle").getString("*"));
                            }
                        }
                        break;
                    }
                }
                
            }
            
            // Check to see if we've reached the end of our token.  If this is
            // the case, then we can end the loop and return what we've got so far.
            if (start >= tLen) {
                loop = false;
                break;
            }
        } while (loop);
        
        return workString.toString();
    }
    
    
}
